uuid = require 'node-uuid'
shared = require './shared'

describe 'nodejs-api-client', () ->
    Client = require '../'
    host = 'localhost'
    port = 5000
    apiKey = uuid.v4()
    secretKey = uuid.v4()

    describe 'constructor', () ->
        it 'should create a new client', () ->
            client = new Client host, port, apiKey, secretKey

            expect(client.host).toEqual host
            expect(client.port).toEqual port
            expect(client.apiKey).toEqual apiKey
            expect(client.secretKey).toEqual secretKey
            expect(client.path).toEqual jasmine.any(Function)

        it 'should create a new client using an object as parameters', () ->
            options =
                host: host
                port: port
                apiKey: apiKey
                secretKey: secretKey

            client = new Client options
            expect(client.host).toEqual host
            expect(client.port).toEqual port
            expect(client.apiKey).toEqual apiKey
            expect(client.secretKey).toEqual secretKey
            expect(client.path).toEqual jasmine.any(Function)

        it 'should create a new client using passing only apiKey and secretKey as parameters', () ->
            options =
                apiKey: apiKey
                secretKey: secretKey
            defaultHost = 'api.openify.it'
            defaultPort = 80

            client = new Client options
            expect(client.host).toEqual defaultHost
            expect(client.port).toEqual defaultPort
            expect(client.apiKey).toEqual apiKey
            expect(client.secretKey).toEqual secretKey
            expect(client.path).toEqual jasmine.any(Function)

        it 'should throw an error if apiKey is missing', () ->
            test = () ->
                options =
                    host: host
                    port: port
                    secretKey: secretKey

                client = new Client options
            expect(test).toThrow()

        it 'should throw an error if secretKey is missing', () ->
            test = () ->
                options =
                    host: host
                    port: port
                    apiKey: apiKey

                client = new Client options
            expect(test).toThrow()

    describe 'requests', () ->
        request = require 'request'
        client = null
        callback = jasmine.createSpy 'callback'

        beforeEach () ->
            options =
                apiKey: apiKey
                secretKey: secretKey
            client = new Client options
            spyOn client, 'request'

        describe 'get', () ->
            describe 'path', () ->
                it 'should call request with "GET" method without options', () ->
                    client.path 'GET', '/datasets', callback
                    expectedArgs = [
                        {
                            qs:
                                apiKey: apiKey
                                secretKey: secretKey
                            url: 'http://api.openify.it:80/datasets'
                            method: 'GET'
                            headers:
                                host: 'api.openify.it'
                        }
                        callback
                    ]
                    expect(client.request.callCount).toEqual 1
                    expect(client.request.mostRecentCall.args).toEqual expectedArgs

                it 'should call request with "GET" method and with options', () ->
                    options =
                        qs:
                            q: 'awesome search query'
                    client.path 'GET', '/datasets', options, callback
                    expectedArgs = [
                        {
                            qs:
                                apiKey: apiKey
                                secretKey: secretKey
                                q: 'awesome search query'
                            url: 'http://api.openify.it:80/datasets'
                            method: 'GET'
                            headers:
                                host: 'api.openify.it'
                        }
                        callback
                    ]
                    expect(client.request.callCount).toEqual 1
                    expect(client.request.mostRecentCall.args).toEqual expectedArgs

            shared.get 'applications'
            shared.get 'datasets'
            shared.get 'organizations'
            shared.get 'users'
