uuid = require 'node-uuid'

module.exports = {}
module.exports.get = (target) ->
    describe 'target', () ->
        callback = jasmine.createSpy 'callback'
        Client = require '../'
        client = null
        host = 'localhost'
        port = 5000
        apiKey = uuid.v4()
        secretKey = uuid.v4()

        beforeEach () ->
            options =
                apiKey: apiKey
                secretKey: secretKey
            client = new Client options
            spyOn client, 'request'

        it "should call get #{target} without options", () ->
            client[target] callback
            expectedArgs = [
                {
                    qs:
                        apiKey: apiKey
                        secretKey: secretKey
                    url: "http://api.openify.it:80/#{target}"
                    method: 'GET'
                    headers:
                        host: 'api.openify.it'
                }
                callback
            ]
            expect(client.request.callCount).toEqual 1
            expect(client.request.mostRecentCall.args).toEqual expectedArgs

        it "should call get #{target} with options", () ->
            options =
                qs:
                    q: 'awesome search query'
            client[target] options, callback
            expectedArgs = [
                {
                    qs:
                        apiKey: apiKey
                        secretKey: secretKey
                        q: 'awesome search query'
                    url: "http://api.openify.it:80/#{target}"
                    method: 'GET'
                    headers:
                        host: 'api.openify.it'
                }
                callback
            ]
            expect(client.request.callCount).toEqual 1
            expect(client.request.mostRecentCall.args).toEqual expectedArgs

        it 'should call get dataset by id without options', () ->
            id = uuid.v4()
            client[target] id, callback
            expectedArgs = [
                {
                    qs:
                        apiKey: apiKey
                        secretKey: secretKey
                    url: "http://api.openify.it:80/#{target}/#{id}"
                    method: 'GET'
                    headers:
                        host: 'api.openify.it'
                }
                callback
            ]
            expect(client.request.callCount).toEqual 1
            expect(client.request.mostRecentCall.args).toEqual expectedArgs

        it 'should call get dataset by id with options', () ->
            id = uuid.v4()
            options =
                qs:
                    fields: 'name'
            client[target] id, options, callback
            expectedArgs = [
                {
                    qs:
                        apiKey: apiKey
                        secretKey: secretKey
                        fields: 'name'
                    url: "http://api.openify.it:80/#{target}/#{id}"
                    method: 'GET'
                    headers:
                        host: 'api.openify.it'
                }
                callback
            ]
            expect(client.request.callCount).toEqual 1
            expect(client.request.mostRecentCall.args).toEqual expectedArgs
